#!/usr/bin/env python

# Script per la ricerca e la riparazione di file mp3 corrotti
# Lo script richiede l'installazione dei seguenti pacchetti:
# ffmpeg
# mp3val

import os
import subprocess

# Definisci il file di log
logfile = "corrupted_mp3.log"

# Svuota il file di log se esiste
with open(logfile, "w") as log:
    log.write("")

# Lista dei file corrotti
corrupted_files = []

# Funzione per stampare il testo in rosso
def print_red(text):
    print("\033[91m" + text + "\033[0m")

# Trova e verifica tutti i file .mp3 nella directory corrente e sottodirectory
for root, dirs, files in os.walk("."):
    for file in files:
        if file.endswith(".mp3"):
            filepath = os.path.join(root, file)
            # Esegui il comando ffmpeg e cattura sia stdout che stderr
            result = subprocess.run(
                ["ffmpeg", "-v", "error", "-i", filepath, "-f", "null", "-"],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE
            )
            # Controlla se il comando è stato eseguito correttamente e se non ci sono errori nell'output
            if result.returncode == 0 and not result.stderr:
                print(f"{file} ok")
            else:
                # Stampare il messaggio di errore nel terminale in rosso
                print_red(f"{file} corrotto")
                # Aggiungi il nome del file al file di log
                with open(logfile, "a") as log:
                    log.write(filepath + "\n")
                # Aggiungi il file alla lista dei file corrotti
                corrupted_files.append(filepath)

# Se sono stati trovati file corrotti, chiedi all'utente se vuole tentarne la riparazione
if corrupted_files:
    print_red("Sono stati trovati i seguenti file corrotti:")
    for file in corrupted_files:
        print(file)
    
    response = input("Vuoi tentare di ripararli con mp3val? (s/n): ").strip().lower()
    if response == 's':
        for file in corrupted_files:
            print(f"Tentando di riparare {file}...")
            repair_result = subprocess.run(
                ["mp3val", "-f", "-nb", file],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE
            )
            if repair_result.returncode == 0:
                print(f"{file} riparato con successo.")
            else:
                print_red(f"Errore nella riparazione di {file}.")
    else:
        print("Riparazione dei file corrotti annullata.")

print(f"Controllo completato. Vedi {logfile} per i dettagli.")


